<?php

namespace Drupal\drm\Plugin\EntityReferenceSelection;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Entity\EntityReferenceSelection\SelectionPluginBase;
use Drupal\drm\Ajax\UpdateDrmOptionsCommand;
use Drupal\drm\Util\DRMUtil;
use Drupal\Core\Database\Query\SelectInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\Core\Entity\EntityReferenceSelection\SelectionInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\views\ViewExecutable;
use Drupal\views\Views;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Plugin override of the 'selection' entity_reference.
 *
 * @package Drupal\drm\Plugin\EntityReferenceSelection
 *
 * @EntityReferenceSelection(
 *   id = "drm_view",
 *   label = @Translation("Dependant Reference: Make field dependent on other field"),
 *   group = "drm_view",
 *   weight = 10
 * )
 */
class DRMViewsSelection extends SelectionPluginBase implements SelectionInterface, ContainerFactoryPluginInterface {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The entity manager.
   *
   * @var \Drupal\Core\Entity\EntityManagerInterface
   */
  protected $entityManager;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The Business Rules Util.
   *
   * @var \Drupal\drm\Util\DRMUtil
   */
  protected $util;

  /**
   * The loaded View object.
   *
   * @var \Drupal\views\ViewExecutable
   */
  protected $view;


  /**
   * Constructs a new SelectionBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityManagerInterface $entity_manager
   *   The entity manager service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\drm\Util\DRMUtil $util
   *   The DRMUtil.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityManagerInterface $entity_manager, ModuleHandlerInterface $module_handler, AccountInterface $current_user, DRMUtil $util) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->entityManager = $entity_manager;
    $this->moduleHandler = $module_handler;
    $this->currentUser = $current_user;
    $this->util = $util;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity.manager'),
      $container->get('module_handler'),
      $container->get('current_user'),
      $container->get('drm.util')
    );
  }


  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'target_type' => NULL,
      'handler' => $this->getPluginId(),
      'entity' => NULL,
      'parent_field' => NULL,
      'fallback_field' => NULL,
    ];
  }

  /**
   * Element validate; Check View is valid.
   */
  public static function settingsFormValidate($element, FormStateInterface $form_state, $form) {
    // Split view name and display name from the 'view_and_display' value.
    if (!empty($element['view_and_display']['#value'])) {
      list($view, $display) = explode(':', $element['view_and_display']['#value']);
    }
    else {
      $form_state->setError($element, t('The views entity selection mode requires a view.'));
      return;
    }

    // Explode the 'arguments' string into an actual array. Beware, explode()
    // turns an empty string into an array with one empty string. We'll need an
    // empty array instead.
    $arguments_string = trim($element['arguments']['#value']);
    if ($arguments_string === '') {
      $arguments = [];
    }
    else {
      // array_map() is called to trim whitespaces from the arguments.
      $arguments = array_map('trim', explode(',', $arguments_string));
    }

    $value = [
      'view_name' => $view,
      'display_name' => $display,
      'arguments' => $arguments,
      'parent_field' => $element['parent_field']['#value'],
      'fallback_field' => $element['fallback_field']['#value']
    ];
    $form_state->setValueForElement($element, $value);
  }


  // Configuration
  //============================================================================
  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $handler_settings = $this->getHandlerSettings();
    $view_settings = !empty($handler_settings['drm_view']) ? $handler_settings['drm_view'] : [];
    $displays = Views::getApplicableViews('entity_reference_display');
    // Filter views that list the entity type we want, and group the separate
    // displays by view.
    $entity_type = $this->entityManager->getDefinition($this->configuration['target_type']);
    $view_storage = $this->entityManager->getStorage('view');

    $options = [];
    foreach ($displays as $data) {
      list($view_id, $display_id) = $data;
      $view = $view_storage->load($view_id);
      if (in_array($view->get('base_table'), [
        $entity_type->getBaseTable(),
        $entity_type->getDataTable(),
      ])) {
        $display = $view->get('display');
        $options[$view_id . ':' . $display_id] = $view_id . ' - ' . $display[$display_id]['display_title'];
      }
    }

    if ($options) {

      // The value of the 'view_and_display' select below will need to be split
      // into 'view_name' and 'view_display' in the final submitted values, so
      // we massage the data at validate time on the wrapping element (not
      // ideal).
      $form['drm_view']['#element_validate'] = [
        [ get_called_class(), 'settingsFormValidate' ],
      ];


      $default_parent = !empty($view_settings['parent_field']) ? $view_settings['parent_field'] : NULL;
      $default_fallback = !empty($view_settings['fallback_field']) ? $view_settings['fallback_field'] : NULL;

      $fields_options = [];

      /** @var \Drupal\field\Entity\FieldConfig $field_config */
      if ($field_config = $this->util->request->get('field_config')) {

        $form['drm_view']['help']['#markup'] = t('This plugin do not works for autocomplete form widget. Make sure you have selected "Select list" or "Check boxes/radio buttons" at "Manage form display" tab.');

        $entity_type = $field_config->getTargetEntityTypeId();
        $bundle = $field_config->getTargetBundle();
        $fields = $this->util->getBundleEditableFields($entity_type, $bundle);


        if (count($fields)) {
          foreach ($fields as $key => $name) {
            // Do not include the dependent field itself.
            if ($key !== 'title' && $key !== $field_config->getName()) {
              $fields_options[$key] = $name;
            }
          }
        }

      }

      /** @var \Drupal\views_ui\ViewUI $view_ui */
      elseif ($view_ui = $this->util->request->get('view')) {
        $form['drm_view']['help']['#markup'] = t('This plugin do not works for autocomplete form widget. Make sure you have selected "Select list" below in this configuration form.');

        /** @var \Drupal\views\ViewExecutable $executable */
        $executable = $view_ui->getExecutable();
        $view = $executable->storage;
        $combined_filters = [];
        foreach ($view->get('display') as $key => $display) {
          $combined_filters += isset($display['display_options']['filters']) ?
            $display['display_options']['filters'] : [];
        }

        foreach ($combined_filters as $filter_key => $filter) {
          if (!isset($filter['exposed']) || $filter['exposed'] == FALSE || $filter['widget'] != 'select') { continue; }
          if ($filter_key == $this->util->request->get('id')) { continue; }

          $fields_options[$filter_key] = $filter['expose']['label'] ?: $filter_key;
        }
      }

      $default = !empty($view_settings['view_name']) ?
        $view_settings['view_name'] . ':' . $view_settings['display_name'] :
        (!empty($view_settings['view_and_display']) ? $view_settings['view_and_display'] : NULL);

      $form['drm_view']['view_and_display'] = [
        '#type' => 'select',
        '#title' => $this->t('View used to select the entities'),
        '#required' => TRUE,
        '#options' => $options,
        '#default_value' => $default,
        '#description' => '<p>' . $this->t('Choose the view and display that select the entities that can be referenced.<br />Only views with a display of type "Entity Reference" are eligible.') . '</p>',
      ];

      $form['drm_view']['parent_field'] = [
        '#type' => 'select',
        '#title' => t('Parent field'),
        '#options' => $fields_options,
        '#required' => TRUE,
        '#description' => t('The field which this field depends. When the parent field value is changed, the available options for this field will be updated using the parent field value as the first argument followed by any particular other argument imputed in the "Views arguments".'),
        '#default_value' => $default_parent,
      ];

      $form['drm_view']['fallback_field'] = [
        '#type' => 'select',
        '#title' => t('Fallback field'),
        '#options' => ['_none' => '- None -'] + $fields_options,
        '#description' => t('The field which this field will fallback depending on if a parent field is absent.'),
        '#default_value' => $default_fallback,
      ];

      $default = !empty($view_settings['arguments']) ? implode(', ', $view_settings['arguments']) : '';
      $form['drm_view']['arguments'] = [
        '#type' => 'textfield',
        '#title' => $this->t('View arguments'),
        '#default_value' => $default,
        '#required' => FALSE,
        '#description' => $this->t('Provide a comma separated list of arguments to pass to the view.'),
      ];
    }
    else {
      if ($this->currentUser->hasPermission('administer views') && $this->moduleHandler->moduleExists('views_ui')) {
        $form['drm_view']['no_view_help'] = [
          '#markup' => '<p>' . $this->t('No eligible views were found. <a href=":create">Create a view</a> with an <em>Entity Reference</em> display, or add such a display to an <a href=":existing">existing view</a>.',
              [
                ':create' => Url::fromRoute('views_ui.add')->toString(),
                ':existing' => Url::fromRoute('entity.view.collection')
                  ->toString(),
              ]) . '</p>',
        ];
      }
      else {
        $form['drm_view']['no_view_help']['#markup'] = '<p>' . $this->t('No eligible views were found.') . '</p>';
      }
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {}

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {}


  // Logic
  //============================================================================
  /**
   * {@inheritdoc}
   */
  public function getReferenceableEntities($match = NULL, $match_operator = 'CONTAINS', $limit = 0) {
    $handler_settings = $this->getHandlerSettings();
    $display_name = $handler_settings['drm_view']['display_name'];
    $args = $handler_settings['drm_view']['arguments'];

    $arguments = $this->buildArguments($args);

    $result = [];
    if ($this->initializeView($match, $match_operator, $limit)) {
      // Get the results.
      $result = $this->view->executeDisplay($display_name, $arguments);
    }

    $return = [];
    if ($result) {
      foreach ($this->view->result as $row) {
        $entity = $row->_entity;
        $return[$entity->bundle()][$entity->id()] = $this->entityManager->getTranslationFromContext($entity)->label();
      }
    }

    return $return;
  }

  /**
   * {@inheritdoc}
   */
  public function countReferenceableEntities($match = NULL, $match_operator = 'CONTAINS') {
    $this->getReferenceableEntities($match, $match_operator);

    return $this->view->pager->getTotalItems();
  }

  /**
   * {@inheritdoc}
   */
  public function validateReferenceableEntities(array $ids) {
    $handler_settings = $this->getHandlerSettings();
    $display_name = $handler_settings['drm_view']['display_name'];
    $args = $handler_settings['drm_view']['arguments'];

    $arguments = $this->buildArguments($args);

    $result = [];
    $ids = $this->getValidIds($arguments);
    if ($this->initializeView(NULL, 'CONTAINS', 0, $ids)) {
      // Get the results.
      $entities = $this->view->executeDisplay($display_name, $arguments);
      $result = is_array($entities) ? array_keys($entities) : [];
    }

    return $result;
  }

  /**
   * {@inheritdoc}
   */
  public function entityQueryAlter(SelectInterface $query) { }


  /**
   * Initializes a view.
   *
   * @param string|null $match
   *   (Optional) Text to match the label against. Defaults to NULL.
   * @param string $match_operator
   *   (Optional) The operation the matching should be done with. Defaults
   *   to "CONTAINS".
   * @param int $limit
   *   Limit the query to a given number of items. Defaults to 0, which
   *   indicates no limiting.
   * @param array|null $ids
   *   Array of entity IDs. Defaults to NULL.
   *
   * @return bool
   *   Return TRUE if the view was initialized, FALSE otherwise.
   */
  protected function initializeView($match = NULL, $match_operator = 'CONTAINS', $limit = 0, $ids = NULL) {
    $handler_settings = $this->getHandlerSettings();
    $view_name = $handler_settings['drm_view']['view_name'];
    $display_name = $handler_settings['drm_view']['display_name'];

    // Check that the view is valid and the display still exists.
    $this->view = Views::getView($view_name);
    if (!$this->view || !$this->view->access($display_name)) {
      drupal_set_message(t('The reference view %view_name cannot be found.', ['%view_name' => $view_name]), 'warning');

      return FALSE;
    }
    $this->view->setDisplay($display_name);

    // Pass options to the display handler to make them available later.
    $entity_reference_options = [
      'match' => $match,
      'match_operator' => $match_operator,
      'limit' => $limit,
      'ids' => $ids,
    ];
    $this->view->displayHandlers->get($display_name)
      ->setOption('entity_reference_options', $entity_reference_options);

    return TRUE;
  }

  /**
   * Return valid ids for validation.
   *
   * @param string $parent_field_value
   *   The parent field value.
   *
   * @return array
   *   Array with valid ids.
   */
  protected function getValidIds($parent_field_value) {
    $handler_settings = $this->getHandlerSettings();
    $display_name = $handler_settings['drm_view']['display_name'];
    $args = $handler_settings['drm_view']['arguments'];

    $arguments = $this->buildArguments($args);

    $result = [];
    if ($this->initializeView(NULL, 'CONTAINS', 0)) {
      // Get the results.
      $result = $this->view->executeDisplay($display_name, $arguments);
    }
    $return = [];
    if ($result) {
      foreach ($this->view->result as $row) {
        $entity = $row->_entity;
        $return[] = $entity->id();
      }
    }

    return $return;
  }

  /**
   * Get the parent field value.
   *
   * @param \Drupal\Core\Entity\Entity|NULL $entity
   *   The fallback entity to extract the value from.
   *
   * @return mixed
   *   The parent field value.
   */
  protected function getParentFieldValue(EntityInterface $entity = NULL) {
    $handler_settings = $this->getHandlerSettings();
    $field = $handler_settings['drm_view']['parent_field'];
    $value = $this->util->request->get($field);

    if (!$value && $entity) {
      $value = $entity->get($field)->getString();
    }

    if (is_array($value) && !empty($value[0]['target_id']) && preg_match('/\((\d+)\)$/', $value[0]['target_id'], $matches)) {
      // If the field widget is entity autocomplete, the returned value is a
      // string which contains the entity id.
      $value = $matches[1];
    }

    return $value;
  }

  /**
   * Get the fallback field value.
   *
   * @param \Drupal\Core\Entity\Entity|NULL $entity
   *   The fallback entity to extract the value from.
   *
   * @return mixed
   *   The fallback field value.
   */
  protected function getFallbackFieldValue(EntityInterface $entity = NULL) {
    $handler_settings = $this->getHandlerSettings();
    $field = isset($handler_settings['drm_view']['fallback_field']) && $handler_settings['drm_view']['fallback_field'] != '_none' ?
      $handler_settings['drm_view']['fallback_field'] : NULL;
    if ($field) {
      $value = $this->util->request->get($field);

      if (!$value && $entity) {
        $value = $entity->get($field)->getString();
      }

      if (is_array($value) && !empty($value[0]['target_id']) && preg_match('/\((\d+)\)$/', $value[0]['target_id'], $matches)) {
        // If the field widget is entity autocomplete, the returned value is a
        // string which contains the entity id.
        $value = $matches[1];
      }

      return $value;
    }

    return NULL;
  }

  /**
   * Get the handler settings.
   *
   * @return array
   *   The handler settings.
   */
  protected function getHandlerSettings() {
    // The ['handler_settings'] was removed on Drupal 8.4. the code bellow is
    // kept for back compatibility. @see https://www.drupal.org/node/2870971
    if (isset($this->configuration['handler_settings'])) {
      return $this->configuration['handler_settings'];
    }

    return $this->configuration;
  }


  private function buildArguments($args = []) {

    // Check if this is a view Route and this is used as exposed filter:
    $routeName = \Drupal::routeMatch()->getRouteName();
    $parent = $fallback = NULL;

    if (strpos($routeName, 'view.') === 0) {
      list ($null, $view_name, $display_name) = explode(".", $routeName);
      $view_executable = Views::getView($view_name);
      $view_executable->setDisplay($display_name);
      $view_executable->preExecute();

      $exposed_map = [];
      /** @var \Drupal\views\Plugin\views\filter\FilterPluginBase[] $filters */
      $filters = $view_executable->filter;
      foreach ($filters as $filter_key => $filter) {
        $options = $filter->options;
        if (!isset($options['exposed']) || $options['exposed'] == FALSE || empty($options['widget']) || $options['widget'] != 'select') { continue; }

        $field_key = isset($options['expose']['identifier']) && $options['expose']['identifier'] != '' ?
          $options['expose']['identifier'] : $options['id'];
        $exposed_map[ $options['id'] ] = $field_key;
      }

      $handler_settings = $this->configuration['drm_view'];
      $get_query = $this->util->request->query;
      foreach ($exposed_map as $real_field_key => $field_key) {
        if ($real_field_key == $handler_settings['parent_field'] && $get_val = $get_query->get($field_key)) {
          $parent = $get_val == '_none' ? NULL : $get_val;
        }
        if ($real_field_key == $handler_settings['fallback_field'] && $get_val = $get_query->get($field_key)) {
          $fallback = $get_val == '_none' ? NULL : $get_val;;
        }
      }
    }
    elseif ($entity = $this->configuration['entity']) {
      $parent_field_value = $this->getParentFieldValue($entity);
      if (is_array($parent_field_value) && !empty($parent_field_value['target_id']) && preg_match('/\((\d+)\)$/', $parent_field_value['target_id'], $matches)) {
        // If the field widget is entity autocomplete, the returned value is a
        // string which contains the entity id.
        $parent_field_value = $matches[1];
      }
      $parent   = !empty($parent_field_value) ? $parent_field_value : NULL;

      $fallback_field_value = $this->getFallbackFieldValue($entity);
      if (is_array($fallback_field_value) && !empty($fallback_field_value['target_id']) && preg_match('/\((\d+)\)$/', $fallback_field_value['target_id'], $matches)) {
        // If the field widget is entity autocomplete, the returned value is a
        // string which contains the entity id.
        $fallback_field_value = $matches[1];
      }
      $fallback = !empty($fallback_field_value) ? $fallback_field_value : NULL;

      if (!$parent && $fallback) {
        $parent = 'all';
      }
      if ($parent && !$fallback) {
        $fallback = 'all';
      }
    }

    return array_merge([$parent], [$fallback], $args);
  }


  /**
   * Update the dependent field options.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return mixed
   *   The updated field.
   */
  public static function updateDependentField(array $form, FormStateInterface $form_state) {
    $trigger_field = $form_state->getTriggeringElement();

    // Update children.
    $children = $trigger_field['#ajax']['drm_children'];
    $response = new AjaxResponse();
    foreach ($children as $child) {

      $handle_settings = NULL;
      $empty_key = '_none';
      $empty_label = t('- None -');
      $exposed_child_key = NULL;
      $entity = NULL;
      $view_executable = NULL;
      $this_field_key = NULL;

      // It's an entity form:
      if ($form_state->getFormObject() && method_exists($form_state->getFormObject(), 'getEntity') && $entity = $form_state->getFormObject()->getEntity()) {
        /** @var \Drupal\Core\Field\FieldDefinitionInterface[] $field_definition */
        $field_definitions = $entity->getFieldDefinitions();

        if (isset($field_definitions[$child]) && $field_definitions[$child]->getSetting('handler') == 'drm_view') {
          $handle_settings = $field_definitions[$child]->getSetting('handler_settings');
          $empty_label = $field_definitions[$child]->isRequired() ? t('- Select a value -') : $empty_label;
        }

        $parent_key   = $handle_settings['drm_view']['parent_field'];
        $parent_field = $form_state->getValue($parent_key, [['target_id' => 'all']]);

        $fallback_key = $handle_settings['drm_view']['fallback_field'];
        $fallback_field = $fallback_key? $form_state->getValue($fallback_key, [['target_id' => 'all']]) : NULL;

        $this_field_key = $child;
      }

      /** @var \Drupal\views\ViewExecutable $view */
      elseif ($view_executable = $form_state->get('view')) {
        $empty_key = 'All';
        $empty_label = t('- Any -');

        $exposed_map = [];
        /** @var \Drupal\views\Plugin\views\filter\FilterPluginBase[] $filters */
        $filters = $view_executable->filter;
        foreach ($filters as $filter_key => $filter) {
          $options = $filter->options;
          if (!isset($options['exposed']) || $options['exposed'] == FALSE || $options['widget'] != 'select') { continue; }
          $field_key = isset($options['expose']['identifier']) && $options['expose']['identifier'] != '' ?
            $options['expose']['identifier'] : $options['id'];
          $exposed_map[ $options['id'] ] = $field_key;
        }

        foreach ($filters as $filter_key => $filter) {
          $options = $filter->options;
          if (!isset($options['exposed']) || $options['exposed'] == FALSE || $options['widget'] != 'select') { continue; }
          if ($filter_key != $child) { continue; }

          $handle_settings = $options['handler_settings'];
          $this_field_key = isset($options['expose']['identifier']) && $options['expose']['identifier'] != '' ?
            $options['expose']['identifier'] : $options['id'];

          $parent_key   = $exposed_map[$handle_settings['drm_view']['parent_field']];
          $parent_get_value = \Drupal::request()->query->get($parent_key, 'all');
          $parent_field = [['target_id' => isset($_POST[$parent_key]) ? $_POST[$parent_key] : $parent_get_value]];

          $fallback_key   = $exposed_map[$handle_settings['drm_view']['fallback_field']];
          $fallback_get_value = \Drupal::request()->query->get($fallback_key, 'all');
          $fallback_field = [['target_id' => isset($_POST[$fallback_key]) ? $_POST[$fallback_key] : $fallback_get_value]];

          break;
        }
      }


      // View execution and Ajax response
      if ($handle_settings && $parent_key) {
        $parent_context_id = $parent_field[0]['target_id'];
        $fallback_context_id = isset($fallback_field[0]) && $fallback_field[0]['target_id'] != "_none" ? $fallback_field[0]['target_id'] : NULL;

        $args = array_merge([$parent_context_id], [$fallback_context_id], $handle_settings['drm_view']['arguments']);

        $view_id = $handle_settings['drm_view']['view_name'];
        $display_id = $handle_settings['drm_view']['display_name'];

        // Get values from the view.
        $view = Views::getView($view_id);
        $view->setArguments($args);
        $view->setDisplay($display_id);
        $view->preExecute();
        $view->build();

        $options = [];
        $options[] = [
          'key' => $empty_key,
          'value' => $empty_label,
        ];

        if ($view->execute()) {
          foreach ($view->result as $resultRow) {
            $options[] = [
              'key'   => $resultRow->_entity->id(),
              'value' => $resultRow->_entity->label(),
            ];
          }
        }

        // Response building:
        $delimiter = '--';
        if ($entity) {
          $form_field = $form[$this_field_key];
          //$form_field['widget']['#options'] = $options;
          if (count($options) === $form_field['widget']['#required'] ? 0 : 1) {
            $form_field['widget']['#access'] = FALSE;
          }
          $delimiter = '-wrapper-';
        }
        elseif ($view_executable) {
          $form_field = $form[$this_field_key];
          //$form_field['#options'] = $options;
          if (count($options) === ($form_field['#required'] ? 0 : 1)) {
            $form_field['#attributes']['disabled'] = 'disabled';
          }
        }

        $html_field_id = explode($delimiter, $form_field['#id'])[0];
        $response->addCommand(new UpdateDrmOptionsCommand($html_field_id, $options));

      }
    }

    return $response;
  }
}
